{
***********************************************************************************
Wobble board exergame for the Parallax HYDRA V1.0
(C)2007 Bryan Bergeron, www.archetypetech.com
Started 2.15.07

Code segments/concepts borrowed from the Graphics_Demo and Memsic_Demo by Parallax,
Matthew Kanwisher's Sub-Hunt demo, and Andre' LaMothe's excellent text
"Game Programming for the Propeller Powered HYDRA"

Note crystal frequency, video pins, and mapping of Propeller pins to Memsic I/O.
Assumes a Memsic 2125 and 'wobble board' as per accompanying Nuts & Volts article

Suggested additions: Music; Score sounds; Multiple levels;Random velocity at start;
Remote restart button or key from position
***********************************************************************************
}

CON

  _clkmode          = xtal1 + pll8x              'note PLL value
  _xinfreq          = 10_000_000 + 0000          'note crystal frequency of 10MHz
  _stack            = ($3000 + $3000 + 100) >> 2 'accomodate display memory and stack

  x_tiles           = 16                         'for display
  y_tiles           = 12                         'for display

  paramcount        = 14       
  bitmap_base       = $2000
  display_base      = $5000

  x_pin             = 22                          'propeller pin P22 = Hydra I/O pin #6
  y_pin             = 23                          'propeller pin P23 = Hydra I/O pin #7

  plot_scale        = 20                          'target size

  'scale dependent on physical configuration of wobble board
  'adjust scale for full range of control with your particular board                   
  wobble_scale      = 12                          'smaller wobble_scale increases sensitivity

  screen_width      = 256                         'position constants 
  screen_height     = 192
  
  score_x_pos       = -screen_width/2 + 10
  score_y_pos       = screen_height/2 - 1*14

  score_val_x_pos   = -screen_width/2 + 10
  score_val_y_pos   = screen_height/2 - 2*14 - 7

  time_x_pos        = screen_width/2 - 10/2*12
  time_y_pos        = screen_height/2 - 1*14

  time_val_x_pos    = screen_width/2 - 10/2*12
  time_val_y_pos    = screen_height/2 - 2*14 - 7

  ticks_divisor     = 15   

VAR

  long  tv_status     '0/1/2 = off/visible/invisible           read-only
  long  tv_enable     '0/? = off/on                            write-only
  long  tv_pins       '%ppmmm = pins                           write-only
  long  tv_mode       '%ccinp = chroma,interlace,ntsc/pal,swap write-only
  long  tv_screen     'pointer to screen (words)               write-only
  long  tv_colors     'pointer to colors (longs)               write-only               
  long  tv_hc         'horizontal cells                        write-only
  long  tv_vc         'vertical cells                          write-only
  long  tv_hx         'horizontal cell expansion               write-only
  long  tv_vx         'vertical cell expansion                 write-only
  long  tv_ho         'horizontal offset                       write-only
  long  tv_vo         'vertical offset                         write-only
  long  tv_broadcast  'broadcast frequency (Hz)                write-only
  long  tv_auralcog   'aural fm cog                            write-only

  word  screen[x_tiles * y_tiles]
  long  colors[64]

  long totalPoints
  long TimeRemaining
  long ticks
  long t_squared      'target hypotenuse
  long i_squared      'tilt hypotenuse

OBJ
  'note tv and gr objects are specific to the Hydra
  
  tv    : "tv_drv_010.spin"
  gr    : "graphics_drv_010.spin"
  acc   : "memsic2125"

PUB start | i, i2, i3,dx, dy, raw, mg, deg, deg2, numchr, target_x, target_y, vel_x_k, vel_y_k

  'start tv
  longmove(@tv_status, @tvparams, paramcount)
  tv_screen := @screen
  tv_colors := @colors
  tv.start(@tv_status)

  'init colors
  repeat i from 0 to 64
    colors[i] := $00001010 * (i+4) & $F + $2B060C02

  'init tile screen
  repeat dx from 0 to tv_hc - 1
    repeat dy from 0 to tv_vc - 1
      screen[dy * tv_hc + dx] := display_base >> 6 + dy + dx * tv_vc + ((dy & $3F) << 10)

  'start and setup graphics
  gr.start
  gr.setup(16, 12, 128, 96, bitmap_base)

  'memsic code **************************************************************************************
  acc.start(x_pin,y_pin)                                'start memsic 2125                                                      
  waitcnt(clkfreq/10 + cnt)                             'wait for things to settle 
  acc.setlevel                                          'assume at startup that the 2125 is level
  '**************************************************************************************************
  
  'init variables for running average
  i        := 0
  i2       := 0
  deg2     := 0

  'init target values
  target_x := 0
  target_y := 0

  'initial target velocity constants; consider using random (?) values here
  'case game mode -> different values
  vel_x_k  := 1
  vel_y_k  := 2

  'countdown at start
  countdown("5")
  countdown("4")
  countdown("3")
  countdown("2")
  countdown("1")

  ticks := 1550                                        '100 seconds
  timeRemaining := ticks / ticks_divisor
  totalPoints := 0                                     'points may be added at startup, depending on tilt
                                   
  repeat
    
    waitcnt(clkfreq/100 + cnt)                          'slow things down a bit
    gr.clear                                            'clear screen

  ' target bounces from screen bounds
    target_x := target_x + vel_x_k
    if (target_x > screen_width/2 - 10) OR (target_x < -screen_width/2 + 10)
      vel_x_k := -vel_x_k

    target_y := target_y + vel_y_k
    if (target_y > screen_height/2 - 10) OR (target_y < -screen_height/2 + 10)
      vel_y_k := -vel_y_k
  
  'target moves within bounds of +/- screen bounds/2 as long as time remains
    if( timeRemaining > 0 ) 
      Draw_target(target_x,target_y,260,660,970)        'x,y,circle1,circle2, circle3
    
  'memsic code **************************************************************************************
    raw := acc.ro                                       'raw acceleration
    mg := raw / wobble_scale                            'convert raw acceleration value to mg's
    if || (i2 - i) > 10                                 'running filter
      i := i2
    i2 := mg / plot_scale                               'scale acceleration value for screen display
    i := (i+i2)/2                                       'remove jitter
    
     if || (deg2 - deg) > 5                             'filter jitter
      deg := deg2
    deg2 := acc.theta >> 19                             'scale 32-bit value to a 13-bit value
    deg := (deg + deg2)/2
  '**************************************************************************************************
    gr.colorwidth(2,12)                                 'wobble board tilt -> move white circle
     if( timeRemaining > 0 )
      gr.arc(0,0, i, i, deg, 0, 1, 0)                      

    gr.colorwidth(2, 0)                                 'Set Color and Width
    gr.textmode(1,1,7,%0000)                            'Set text mode

  'compute and compare hypotenuse of target and white circle.  If equal, assume they overlap
  'consider using a sine lookup table to prevent false scores - but not really necessary  
    t_squared := (target_x * target_x) + (target_y * target_y)
    i_squared := i*i

    if || (i_squared - t_squared) < 550                  'Add a sound here!
      if (timeRemaining >0)
        gr.text(-120,-80,string("Score!"))
        totalPoints := totalPoints +1

    if( timeRemaining > 0 )                               'countdown timer
      ticks := ticks - 1
      timeRemaining := ticks / Ticks_divisor
    else
      gr.text(-95,0,string("Hit Reset for another round"))

    drawscore    
    gr.copy(display_base)                               'copy bitmap to display

PUB Countdown(numchr)
    gr.clear                                            
    gr.textmode (8,8,6,5)
    gr.colorwidth (2,8)
    gr.text (0,0,@numchr)
    gr.copy(display_base)                               
    waitcnt(clkfreq *2 + cnt)         

PUB Draw_target(x,y,Sin1,Sin2,Sin3)
    gr.colorwidth(1, 6)
    gr.arc(x, y, Sin1/plot_scale,Sin1/plot_scale, 0, $100, $21, 2)
    gr.colorwidth(2, 6)
    gr.arc(x, y, Sin2/plot_scale,Sin2/plot_scale, 0, $100, $21, 2)
    gr.colorwidth(3, 6)
    gr.arc(x, y, Sin3/plot_scale,Sin3/plot_scale, 0, $100, $21, 2)

pub DrawScore
   gr.textmode(2,1,5,3)
   gr.colorwidth(2,0)

   gr.text(SCORE_X_POS, SCORE_Y_POS, @score_string)
   gr.text(TIME_X_POS, TIME_Y_POS, @time_string)

   Int_To_String(@value_string, totalPoints)
   gr.text(SCORE_VAL_X_POS, SCORE_VAL_Y_POS, @value_string)

   Int_To_String(@value_string2, timeRemaining)
   gr.text(TIME_VAL_X_POS, TIME_VAL_Y_POS, @value_string2)

PUB Int_To_String(str, i) | t
  str+=5
  repeat t from 0 to 4
    BYTE[str] := 48+(i // 10)
    i/=10   
    str--
DAT
tvparams                long    0               'status
                        long    1               'enable
                        long    %011_0000       'pins
                        long    %0000           'mode
                        long    0               'screen
                        long    0               'colors
                        long    x_tiles         'hc
                        long    y_tiles         'vc
                        long    10              'hx
                        long    1               'vx
                        long    0               'ho
                        long    0               'vo
                        long    60_000_000'_xinfreq<<4  'broadcast
                        long    0               'auralcog

score_string            byte    "Score",0               
time_string             byte    "Time",0                
value_string            byte    "000000",0
value_string2           byte    "000000",0